----------------------------------------------------------------------------
--                                                                        --
-- Copyright (c) 1995-1999 by Papillon Research Corp. All rights reserved.--
--                                                                        --
-- This source file may be used and distributed without restriction       --
-- provided that this copyright statement is not removed from the file    --
-- and that any derivative work contains this copyright notice. This      --
-- code or it's derivatives may not be sold as a whole or as a part of    --
-- any commercial product without written consent from Papillon Research. --
--                                                                        --
-- This VHDL code is distributed in the hope that it will be useful,      --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of         --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                   --
--                                                                        --
-- Program name: amcc5933.vhd                                             --
--							 	          --
----------------------------------------------------------------------------
----------------------------------------------------------------------------
--
--   File name :  s5933.vhd
--   Title     :  AMCC S5933 PCI BUS INTERFACE CHIP
--
--   Roadmap   : 
--   
----------------------------------------------------------------------------
--   Modification History :
--          Date    		Author  	Rev      Comments
-- Wed Feb  1 14:00:00 1995     Papillon        A         Creation.
----------------------------------------------------------------------------
--
-- Device timing from AMCC S5933 Data sheet "Summer 1994"
--                    Plus errata sheet 10/14/94                           
----------------------------------------------------------------------------
--  OPERATION
--
--  This model is designed for use in the Modeltech simulation environment.
--  It requires some signals to be setup befor a cycle is simulated, these
--  are:
--
--    For pass through cycles:
--     PTWR
--     PTBURST
--     PTNUM 
--     PTBE
--     PTADDR
--     PTDATA
--     PT_CYC
--    For fifo data cycles:
--     FILL_PCI_FIFO
--
--  For example to simulate a pass through write to adrs 0xaa you would setup these
--  conditions:
--
--    L PTWR
--    L PTBURST
--    A PTNUM 1\h
--    A PTBE 0\h
--    A PTADDR aa\h
--    A PTDATA 55\h
--
--  Then start a pass through cycle in the model by asserting PT_CYC:
--
--    H PT_CYC
--    sim 400ns
--    L PT_CYC
--
--  Asserting FILL_PCI_FIFO will cause the internal fifo to begin filling with 
--  data from the process pci_dat. This data can be random patterns or taken from
--  a pattern table.
--
--  See the included testbench for alternative method of operation.
--------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL ;
USE work.papio.ALL;
USE std.textio.all;

 ENTITY amcc5933 is
	GENERIC
	       (
                tcl:    TIME    := 30 ns;
		t1: 	TIME	:= 12 ns;
		t2:	TIME    := 12 ns;
		t10:	TIME    :=  3 ns;
		t10a:	TIME    :=  3 ns;
		t11a:	TIME    :=  2 ns;
		t13:	TIME    :=  5 ns;
		t14:	TIME    :=  2 ns;
		t17:	TIME    := 13 ns;
		t18:	TIME    := 12 ns;
		t19:	TIME    :=  4 ns;       -- added 2ns for bus capacitance
		t22:	TIME    := 11 ns;       -- 22 & 23 are changed in
		t22A:	TIME    := 11 ns;       -- erata 10.5 ns each
		t23:	TIME    := 11 ns;       
		t24:	TIME    :=  5 ns;
		t25:	TIME    :=  0 ns;
		t26:	TIME    :=  5 ns;
		t27:	TIME    :=  3 ns;
		t28:	TIME    :=  6 ns;
		t29:	TIME    :=  5 ns;
		t30:	TIME    :=  2 ns;
		t31:	TIME    :=  5 ns;
		t32:	TIME    := 2  ns;
		t33:	TIME    := 15 ns;
		t34:	TIME    := 12 ns;
                FULLCOUNT            : integer := 16;
                FULL_INACTIVE_TICKS  : integer := 4
		);

	PORT
	    (

		-- ADD-ON BUS PINS
		WR:            in       STD_LOGIC;
		RD:            in       STD_LOGIC;
		CS:            in       STD_LOGIC;  -- This is the SELECT line.
		MODE:          in       STD_LOGIC;
		WRFIFO:        in       STD_LOGIC;
		RDFIFO:        in       STD_LOGIC;
		FRC_L:         in       STD_LOGIC;
		FWC_L:         in       STD_LOGIC;
		BE:            in       STD_LOGIC_VECTOR(3 downto 0);
		ADR:           in       STD_LOGIC_VECTOR(4 downto 0);

		DQ:         inout       STD_LOGIC_VECTOR(31 downto 0);

		WRFULL:        out      STD_LOGIC;
		RDEMPTY:       out      STD_LOGIC;


		-- PASS-TROUGH STIMULUS SIGNALS
                pt_cyc:        IN       STD_LOGIC;
                pcontrol:      in       STD_LOGIC_VECTOR(7 downto 0);
                paddr :        in       STD_LOGIC_VECTOR(31 downto 0);
                pdata :        in       STD_LOGIC_VECTOR(31 downto 0);
                fill_pci_fifo: in       STD_LOGIC := '0';
                 
		-- PASS-TROUGH INTERFACE PINS
		PTRDY:         in       STD_LOGIC;
                PTADR:         in       STD_LOGIC;
               
                PTATN:        out       STD_LOGIC;
                PTBURST:      out       STD_LOGIC;
                PTWR:         out       STD_LOGIC;

                PTNUM:        out       STD_LOGIC_VECTOR(1 downto 0);
                PTBE:         out       STD_LOGIC_VECTOR(3 downto 0);

		
		-- SYSTEM PINS
		SYSRST:       BUFFER    STD_LOGIC;  -- "inout" for feedback      
		BPCLK:        BUFFER    STD_LOGIC;  -- "inout" for feedback   
		IRQ:          out       STD_LOGIC;

	
		-- PCI PINS
		CLK:          in        STD_LOGIC;
		RST:          in        STD_LOGIC;
		INTA:        out        STD_LOGIC

           );

 END ; -- entity amcc5933
--------------------------------------------------------------------------------
-- Architecture
--
--
--------------------------------------------------------------------------------
ARCHITECTURE behavior of amcc5933 is

-------------------------------------------------------------------------------
--  CONSTANT DECLARATIONS
-------------------------------------------------------------------------------
  
  CONSTANT      one                      : STD_LOGIC_VECTOR(30 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','1');

  CONSTANT      fours                    : STD_LOGIC_VECTOR(30 downto 0) :=
                ('1','0','0','0','1','0','0','0','1','0','0','0','1','0','0','0',
                 '1','0','0','0','1','0','0','0','1','0','0','0','1','0','0');
                 
  CONSTANT      Z32              : STD_LOGIC_VECTOR(31 downto 0) :=
                ('Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z',
                 'Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z','Z');

  CONSTANT      X32              : STD_LOGIC_VECTOR(31 downto 0) :=
                ('X','X','X','X','X','X','X','X','X','X','X','X','X','X','X','X',
                 'X','X','X','X','X','X','X','X','X','X','X','X','X','X','X','X');

  CONSTANT      bad_dq              : STD_LOGIC_VECTOR(31 downto 0) :=
                ('1','0','1','1','1','0','1','0','1','1','0','1','0','0','0','0',
                 '1','1','0','1','1','0','1','0','1','1','0','1','1','0','1','0');
                 
  -- Some useful constants for timing checks  
  CONSTANT       falling         : STD_LOGIC   := '0';  
  CONSTANT       rising          : STD_LOGIC   := '1'; 
  CONSTANT       SETUP           : INTEGER :=  0 ;
  CONSTANT       HOLD            : INTEGER :=  1 ;
  CONSTANT       PULSE           : INTEGER :=  2 ;
  CONSTANT       META            : INTEGER :=  3 ; 

  -- State Machine states
  CONSTANT      idle             : INTEGER := 0;
  CONSTANT      s1               : INTEGER := 1;
  CONSTANT      s2               : INTEGER := 2;
  CONSTANT      s3               : INTEGER := 3;
  CONSTANT      s4               : INTEGER := 4;
  CONSTANT      s5               : INTEGER := 5;
  CONSTANT      s11              : INTEGER := 11;
  CONSTANT      s12              : INTEGER := 12;
  CONSTANT      s13              : INTEGER := 13;
  CONSTANT      s14              : INTEGER := 14;
  CONSTANT      s15              : INTEGER := 15;

  -- Internal Address locations
  CONSTANT      aomb1            : integer := 16;
  CONSTANT      aomb2            : integer := 20;
  CONSTANT      aomb3            : integer := 24;
  CONSTANT      aomb4            : integer := 28;
  CONSTANT      apta             : integer := 40;
  CONSTANT      aptd             : integer := 44;
  CONSTANT      afifo            : integer := 32;
  CONSTANT      mwar             : integer := 36;
  

-------------------------------------------------------------------------------
--  SIGNAL DECLARATIONS
-------------------------------------------------------------------------------
signal pstate : integer;
signal frd_ptr : integer := 0;
signal fwr_ptr : integer := 0;
    
  SIGNAL        pci_seed                 : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');

  SIGNAL        pci_data                 : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
                 
  SIGNAL        burst_size               : INTEGER := 4;

  -- Internal fifo stuff
  SIGNAL        temp                     : STD_LOGIC_VECTOR(30 downto 0);
  SIGNAL        fifo_DQ                  : STD_LOGIC_VECTOR(31 downto 0);
  TYPE PCI_FIFO_TYPE IS ARRAY (7 DOWNTO 0) OF STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL        pci_fifo                 : PCI_FIFO_TYPE;
  SIGNAL        full_flag                : STD_LOGIC := '0';
  SIGNAL        empty_flag               : STD_LOGIC := '0';

  -- Pass through mode signals
  SIGNAL        pwr             : STD_LOGIC := '1';
  SIGNAL        pburst          : STD_LOGIC := '1';
  SIGNAL        pnum            : STD_LOGIC_VECTOR(1 downto 0)  := "00";
  SIGNAL        pbe             : STD_LOGIC_VECTOR(3 downto 0)  := "0000";

  SIGNAL        iPTADR                   : STD_LOGIC := '1';
  SIGNAL        reg_cyc                  : STD_LOGIC := '0';
  SIGNAL        tc_en_set                : STD_LOGIC := '0';  -- Check setup violations
  SIGNAL        tc_en_hold               : STD_LOGIC := '0';  -- Check hold violations
  SIGNAL        rand_pat                 : STD_LOGIC := '0';
  SIGNAL        INFO                     : STD_LOGIC:= '1';     -- Debug flag   
  SIGNAL        pt_cycl                  : STD_LOGIC := '0';
  SIGNAL        pcyc                     : STD_LOGIC := '0';
  SIGNAL        wrfull_i                 : STD_LOGIC := '0';
  SIGNAL        bpclk_i                  : STD_LOGIC := '1';
  SIGNAL        addr_ux01                : STD_LOGIC_VECTOR(4 DOWNTO 0);
  SIGNAL        be_ux01                  : STD_LOGIC_VECTOR(3 DOWNTO 0);
  SIGNAL        rd_ux01                  : STD_LOGIC;
  SIGNAL        rdfifo_ux01              : STD_LOGIC;
  SIGNAL        wrfifo_ux01              : STD_LOGIC;
  SIGNAL        wr_ux01                  : STD_LOGIC;
  SIGNAL        dq_ux01                  : STD_LOGIC_VECTOR(31 DOWNTO 0);  
  SIGNAL        cs_ux01                  : STD_LOGIC;
  SIGNAL        ptrdy_ux01               : STD_LOGIC;



  
-------------------------------------------------------------------------------
--  INTERNAL REGISTERS
-------------------------------------------------------------------------------

   SIGNAL       mail1_DQ          : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
   SIGNAL       mail2_DQ          : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
   SIGNAL       mail3_DQ          : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
   SIGNAL       mail4_DQ          : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
   SIGNAL       MRAR             : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
   SIGNAL       MRTC             : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
   SIGNAL       pt_DQ            : STD_LOGIC_VECTOR(31 downto 0) :=
                ('1','0','0','0','0','1','1','1','0','1','1','0','0','1','0','1',
                 '0','1','0','0','0','0','1','1','0','0','1','0','0','0','0','1');
                 
   SIGNAL       pt_addr          : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');

   SIGNAL        mwar_DQ         : STD_LOGIC_VECTOR(31 downto 0) :=
                ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
                 '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');


BEGIN

--------------------------------------------------------------------------------
-- Combinatorial stuff
--
--------------------------------------------------------------------------------
pwr    <= pcontrol(0);
pburst <= pcontrol(1);
pnum   <= pcontrol(3 downto 2); 
pbe    <= pcontrol(7 downto 4);  

bpclk_i    <= not bpclk_i AFTER 15 ns;     -- Create the clock directly
BPCLK      <= bpclk_i;
SYSRST     <= RST ;
tc_en_set  <= SYSRST;                    -- Do timing checks if not RESET.
tc_en_hold <= SYSRST;                    -- Do timing checks if not RESET.
temp       <= pci_data(30 downto 0);
RDEMPTY    <= empty_flag;
WRFULL     <= wrfull_i;

-------------------------------------------------------------------------------
--  Process:   pci_dat
--  Purpose:   To provide a semi random stream of data to the model as if it
--             were coming from the PCI bus. The random function produces a
--             psuedo random number from the seed. Different simulation passes
--             can have different seeds by changing the value of pci_data at
--             any time.
--             Setting the variable "rand_pat" to a one causes the pci_data
--             pattern to come from the random data pattern generator, else
--             the pci_data comes from an array of patterns.
-------------------------------------------------------------------------------
pci_dat : PROCESS(BPCLK)

  VARIABLE      i        : integer := 0;
  
BEGIN  --  PROCESS pci_data 

  IF (BPCLK = '1' AND BPCLK'EVENT) THEN

   if fill_pci_fifo = '1' then
--     pci_data <= pdata;
   elsIF (pt_cyc = '1' AND pt_cycl = '0') THEN
     pcyc <= '1';
     pt_cycl <= '1';
--     pci_data <= pdata;
   END IF;

   IF (PTRDY = '0') THEN
     pcyc <= '0';
   END IF ;
   
   IF (pt_cyc = '0') THEN
     pt_cycl <= '0';
   END IF ;
   
END IF;
 
END Process pci_dat;
pci_data <= pdata;


-------------------------------------------------------------------------------
--  Process:   io_mux
--  Purpose:   This process muxes data to the output pins. In the future, if
--             its needed, it could do byte lane steering and/or endian
--             conversion.
--   
--   This may end up being a good place to put register reads etc
--
-------------------------------------------------------------------------------
io_mux : PROCESS (fifo_DQ,pt_addr,pt_DQ,iPTADR,CS,RD,WR,ADR,RDFIFO,RST,BPCLK)

  VARIABLE      iadr                 : integer := 0;
  VARIABLE      count                : integer := 0;
  VARIABLE      counting             : BOOLEAN := TRUE;
  VARIABLE       outline:         LINE;
  
BEGIN  --  PROCESS io_mux 

  IF (IS_X(ADR) = FALSE) THEN
    iadr := to_int(ADR & '0' & '0');
  END IF;
  
 IF (RDFIFO = '0') THEN
     DQ <= fifo_DQ;
     
 ELSIF  (iPTADR = '0') THEN
     DQ <= pt_addr AFTER t17;

 ELSIF (RD = '0' AND CS = '0') THEN 
     CASE iadr IS
        WHEN apta =>
          DQ <= pt_addr AFTER t17;
          
        WHEN aptd =>
          DQ <= pt_DQ AFTER t17;
          
        WHEN afifo =>
          DQ <= fifo_DQ AFTER t17;
          
        WHEN aomb1 =>
          DQ <= mail1_DQ AFTER t17;
          
        WHEN aomb2 =>
          DQ <= mail2_DQ AFTER t17;
          
        WHEN aomb3 =>
          DQ <= mail3_DQ AFTER t17;
          
        WHEN aomb4 =>
          DQ <= mail4_DQ AFTER t17;
          
        WHEN mwar =>
          DQ <= mwar_DQ AFTER t17;
          
        WHEN OTHERS =>
          DQ <= bad_DQ;
          
     END CASE;                             --  iaddr
 ELSE
   DQ <= Z32 AFTER t18;
 END IF;

 
 ------------------------------------------------------------------------------
 --  Handle normal register writes. Strobe data on rising edge of write(WR)
 ------------------------------------------------------------------------------
 IF (CS = '0' AND WR = '0' AND BPCLK'EVENT AND BPCLK = '1')
 THEN
     CASE iadr IS
       WHEN aomb1 =>
           mail1_DQ <= to_ux01(DQ);
           INTA <= '0', '1' AFTER 66 ns; -- Make PCI int line go low.
           write (OUTLINE, string'("*** MAILBOX INTERRUPT 1 GENERATED *** @ "));
           write (OUTLINE, now);
           writeline(OUTPUT, OUTLINE);

       WHEN aomb2 =>
           mail2_DQ <= to_ux01(DQ);
           INTA <= '0', '1' AFTER 66 ns; -- Make PCI int line go low.
           write (OUTLINE, STRING'("*** MAILBOX INTERRUPT 2 GENERATED *** @ "));
           write (OUTLINE, now);
           writeline(OUTPUT, OUTLINE);

       WHEN aomb3 =>
           mail3_DQ <= to_ux01(DQ);
           INTA <= '0', '1' AFTER 66 ns; -- Make PCI int line go low.
           write (OUTLINE, STRING'("*** MAILBOX INTERRUPT 3 GENERATED *** @ "));
           write (OUTLINE, now);
           writeline(OUTPUT, OUTLINE);

       WHEN aomb4 =>
           mail4_DQ <= to_ux01(DQ);
           INTA <= '0', '1' AFTER 66 ns; -- Make PCI int line go low.
           write (OUTLINE, STRING'("*** MAILBOX INTERRUPT 4 GENERATED *** @ "));
           write (OUTLINE, now);
           writeline(OUTPUT, OUTLINE);

       WHEN mwar =>
           mwar_DQ <= to_ux01(DQ);
           wrfull_i <= '0';
       WHEN OTHERS =>
         NULL;
     END CASE;                           --  iaddr
 END IF;

 IF (BPCLK'EVENT AND BPCLK = '1' AND WRFIFO = '0') THEN
   IF (wrfull_i = '0') THEN 
     mwar_DQ <= to_vec((to_int(mwar_DQ) + 4),32);
   END IF ;
   count := count + 1;
   IF (count = FULLCOUNT) THEN
     wrfull_i <= '1' AFTER t22a, '0' AFTER ((FULL_INACTIVE_TICKS*tcl)+t22a);
     counting := FALSE;
   END IF ;
   IF (wrfull_i = '0' AND counting = FALSE) THEN 
     count := 0;
     counting := TRUE;
   END IF ;
 END IF ;
  
END PROCESS io_mux;

-------------------------------------------------------------------------------
--  Process:   fifo
--  Purpose:   This is the pci fifo part of the model. The process is entered by
--             either version of CLK. In the simulation, raising "fill_pci_fifo"
--             will initiate the PCI side fifo fills.
--
-------------------------------------------------------------------------------
fifo : PROCESS(SYSRST,BPCLK)

 VARIABLE       wr_ptr   : integer := 0;
 VARIABLE       rd_ptr   : integer := 0;
         
BEGIN  --  PROCESS pci_fifo 

  IF (BPCLK = '1' AND BPCLK'EVENT)           -- Do stuff sync'ed to pci_clk
  THEN

    IF (SYSRST = '0' or frc_l = '0') THEN
      wr_ptr := 0;
      rd_ptr := 0;
      empty_flag <= '1';
      full_flag <= '0';
     
    ELSE
      IF (fill_pci_fifo = '1' AND full_flag = '0') THEN
        pci_fifo(wr_ptr) <= pci_data(31 downto 0);
        wr_ptr := (wr_ptr+1) mod 8;
        empty_flag <= '0' AFTER T23;         -- THIS DELAY IS ASSUMED TO BE THE
                                           -- SAME AS THE RISING EDGE DELAY.
        IF (WR_PTR = RD_PTR) THEN
          FULL_FLAG <= '1' AFTER T22;
        ELSE
          FULL_FLAG <= '0';
        END IF;
      END IF;
  
      IF ( RDFIFO = '0' AND empty_flag = '0') THEN
        -- Debug("READING FROM INTERNAL FIFO @ ",INFO);
    
--    fifo_DQ(31 downto 0) <= pci_fifo(rd_ptr) AFTER t33;
        rd_ptr := (rd_ptr + 1) mod 8;
        full_flag <= '0' AFTER t22;          -- This delay is assumed to be the
                                             -- same as the rising edge delay.
        IF (rd_ptr = wr_ptr) THEN
          empty_flag <= '1' AFTER t23;
        ELSE
          empty_flag <= '0';
        END IF;
      END IF;
    END IF;
  END IF;

  IF (BPCLK = '1' AND BPCLK'EVENT)       -- DO STUFF SYNCED TO BUF CLOCK
  THEN
   IF (RDFIFO = '1')
   THEN
     DQ <= Z32 AFTER t34;
   END IF;   
  END IF;                                -- prising BPCLK
--fifo_DQ(31 downto 0) <= pci_fifo(rd_ptr);
--fifo_DQ(31 downto 0) <= pci_fifo(rd_ptr) AFTER t33;
  frd_ptr <= rd_ptr;
  fwr_ptr <= wr_ptr;

END PROCESS fifo;
fifo_DQ(31 downto 0) <= pci_fifo(frd_ptr) AFTER t33;
--fifo_DQ(31 downto 0) <= pci_fifo(frd_ptr) AFTER t33 when empty_flag = '0' else
--                        pci_fifo((frd_ptr + 7) mod 8) AFTER t33;

-------------------------------------------------------------------------------
--  Process:   pass_through
--  Purpose:   State machine to generate pass through cycles.
--
-------------------------------------------------------------------------------
pass_through: PROCESS (BPCLK,PTADR,RST)

   VARIABLE     state            : integer := 0;
   VARIABLE     burst_count      : integer := 1;
   VARIABLE     retry_count      : integer := 1;
   VARIABLE     paddrtemp        : STD_LOGIC_VECTOR(31 DOWNTO 0);
   VARIABLE       outline:         LINE;
BEGIN  --  PROCESS pass_through 

   ----------------------------------------------------------------------------
   --  Reading of the pass through address is not clocked.
   ----------------------------------------------------------------------------
  
   IF (PTADR = '0' AND PTADR'EVENT) THEN
     iPTADR <= '0';
     pt_addr <= paddrtemp;
     
     IF (RD ='0' OR WR='0')
     THEN
       WRITE(OUTLINE, STRING'("ERROR - ILLEGAL PASS THROUGH ADDR CONDITION @"));
       writeline(OUTPUT, OUTLINE);
     END IF;
   END IF;

   IF (PTADR = '1' AND PTADR'EVENT) THEN
     iPTADR <= '1';
     pt_addr <= X32 AFTER t19;
   END IF;

   ----------------------------------------------------------------------------
   --  All other pass through events are synced to BPLCLK rising
   ----------------------------------------------------------------------------

    IF (RST = '0') THEN

       PTATN   <= '1';
       PTBURST <= '1';
       PTWR    <= '1';
       PTNUM   <= ('1','1');
       PTBE    <= ('1','1','1','1');

   ELSIF (BPCLK = '1' AND BPCLK'EVENT) THEN
      
     CASE state IS

       WHEN idle =>

         retry_count := 1;
         burst_count := 1;
         
         IF (pcyc = '1')               -- Start a pass through cycle
         THEN
           -- Debug("Starting Pass Through cycle @",info);
           paddrtemp := paddr;
           IF (pwr = '1')
           THEN                          -- Do a write
             state := s1;
           ELSE                          -- Do a read
             state := s11;
           END IF;

         ELSE
           state := idle;
         END IF;

       WHEN s1 =>

         -- Debug("In Pass Through Write state s1 @",info);
         
         PTWR  <= pwr AFTER t24;
         PTNUM <= pnum AFTER t24;
         state := s2;

       WHEN s2 =>

         -- Debug("In Pass Through Write state s2 @",info);
         
         pt_DQ   <= pci_data;
         PTATN   <= '0' AFTER t24;
         PTBURST <= pburst AFTER t24;
         PTBE    <= pbe AFTER t24;
         state   := s3;

       WHEN s3 =>

         -- Debug("In Pass Through Write state s3 @",info);
         
         IF (PTRDY = '0')
         THEN
           IF (pburst = '0' AND burst_count < burst_size)
           THEN
             paddrtemp := TO_VEC((TO_INT(paddrtemp) + 4),32);
             burst_count := burst_count + 1;
             state := s2;
          
           ELSE
             state := idle;
             PTATN   <= '1' AFTER t25;
             PTBURST <= '1' AFTER t25;
             PTWR    <= '1' AFTER t25;
             PTNUM   <= ('1','1') AFTER t25;
             PTBE    <= ('1','1','1','1') AFTER t25;
             
           END IF;
         ELSE
           IF (retry_count > 8 AND info = '1')
           THEN
            write(OUTLINE, STRING'("***** ATTENTION ****** PCI MASTER HAS BEEN TOLD TO RETRY @: "));
            write (OUTLINE, now);
            writeline(OUTPUT, OUTLINE);
            retry_count := 1;
           END IF;
           retry_count := retry_count + 1;
           state := s3;
         END IF;

       ------------------------------------------------------------------------
       --  Do Pass Through Reads
       ------------------------------------------------------------------------

       ------------------------------------------------------------------------
       --  The rev 2 parts have a bug on pass through reads which only asserts
       --  PTATN for two clocks, this change is made offset. normal commented out.
       ------------------------------------------------------------------------
       WHEN  s11 =>

         -- Debug("In Pass Through Read state s1 @",info);
         
         PTATN <= '0' AFTER t24;
         PTWR  <= pwr AFTER t24;
         PTNUM <= pnum AFTER t24;
         state := s12;

       WHEN s12 =>

         -- Debug("In Pass Through Read state s2 @",info);
         
         PTATN   <= '0' AFTER t24;       -- During bursts this goes low
         PTBURST <= pburst AFTER t24;
         PTBE    <= pbe AFTER t24;
         state   := s13;

       WHEN s13 =>

         -- Debug("In Pass Through Read state s3 @",info);

         IF (PTRDY = '0')
         THEN
           IF (pburst = '0' AND burst_count < burst_size)
           THEN
             paddrtemp := TO_VEC((TO_INT(paddrtemp) + 4),32);
             burst_count := burst_count + 1;
             state := s12;
          
           ELSE
             state := idle;
             PTATN   <= '1' AFTER t25;
             PTBURST <= '1' AFTER t25;
             PTWR    <= '1' AFTER t25;
             PTNUM   <= ('1','1') AFTER t25;
             PTBE    <= ('1','1','1','1') AFTER t25;
             
           END IF;
         ELSE
           IF (retry_count = 6 AND info = '1')          -- 6 plus state 12 = 7
           THEN
            WRITE(OUTLINE, STRING'("***** ATTENTION ****** PCI MASTER HAS BEEN TOLD TO RETRY @: "));
            write (OUTLINE, now);
            writeline(OUTPUT, OUTLINE);
            retry_count := 1;
           END IF;
           retry_count := retry_count + 1;
           state := s13;
         END IF;

       WHEN OTHERS =>
         state := idle;
     END CASE;                           --  state

   END IF;
   pstate <= state;
  
END PROCESS pass_through;

addr_ux01   <= to_ux01(ADR);
be_ux01     <= to_ux01(BE);
rd_ux01     <= to_ux01(RD);
rdfifo_ux01 <= to_ux01(RDFIFO);
wrfifo_ux01 <= to_ux01(WRFIFO);
wr_ux01     <= to_ux01(WR);
dq_ux01     <= to_ux01(DQ);
cs_ux01     <= to_ux01(CS);
ptrdy_ux01  <= to_ux01(PTRDY);

timing_checks : PROCESS (CS, BE, ADR, BPCLK, RDFIFO, WRFIFO, RD, WR, DQ, PTRDY)

 VARIABLE       BPCLK_CHNG     : TIME := 0 ns;
 VARIABLE       ADR_CHNG       : TIME := 0 ns;
 VARIABLE       BE_CHNG        : TIME := 0 ns;
 VARIABLE       RD_CHNG        : TIME := 0 ns;
 VARIABLE       RDFIFO_CHNG    : TIME := 0 ns;
 VARIABLE       WRFIFO_CHNG    : TIME := 0 ns;
 VARIABLE       WR_CHNG        : TIME := 0 ns;
 VARIABLE       DQ_CHNG        : TIME := 0 ns;
 VARIABLE       CS_CHNG        : TIME := 0 ns;
 VARIABLE       PTRDY_CHNG     : TIME := 0 ns;
 VARIABLE       outline:         LINE;
  
BEGIN  --  PROCESS timing_checks 

 IF (BPCLK'EVENT AND BPCLK = '1') THEN
   BPCLK_CHNG := now;
 END IF ;
 IF (addr_ux01'event) THEN
   ADR_CHNG := now;
   IF ((now - BPCLK_CHNG) < t14 AND tc_en_hold = '1') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION: BPCLK RISING TO ADR HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;
 IF (be_ux01'event) THEN
   BE_CHNG := now;
   IF ((now - BPCLK_CHNG) < t14 AND tc_en_hold = '1') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION:  BPCLK RISING TO BE HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;
 IF (rd_ux01'event) THEN
   RD_CHNG := now;
   IF ((now - BPCLK_CHNG) < t30 AND tc_en_hold = '1') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION:  BPCLK RISING TO RD HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;
 IF (rdfifo_ux01'event) THEN
   RDFIFO_CHNG := now;
   IF ((now - BPCLK_CHNG) < t30 AND tc_en_hold = '1') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION:  BPCLK RISING TO RDFIFO HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;
 IF (wrfifo_ux01'event) THEN
   WRFIFO_CHNG := now;
   IF ((now - BPCLK_CHNG) < t30 AND tc_en_hold = '1') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION:  BPCLK RISING TO WRFIFO HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;
 IF (wr_ux01'event) THEN
   WR_CHNG := now;
   IF ((now - BPCLK_CHNG) < t30 AND tc_en_hold = '1') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION:  BPCLK RISING TO WR HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;
 IF (dq_ux01'event) THEN
   DQ_CHNG := now;
   IF ((now - BPCLK_CHNG) < t32 AND tc_en_hold = '1' AND WR = '0') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION:  BPCLK RISING TO DQ HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;
 IF (cs_ux01'event) THEN
   CS_CHNG := now;
   IF ((now - BPCLK_CHNG) < t11a AND tc_en_hold = '1') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION:  BPCLK RISING TO CS HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;
 IF (ptrdy_ux01'event) THEN
   PTRDY_CHNG := now;
   IF ((now - BPCLK_CHNG) < t27 AND tc_en_hold = '1') THEN 
     WRITE(OUTLINE, STRING'("TIMING VIOLATION:  BPCLK RISING TO PTRDY HOLD @: "));
     write (OUTLINE, now);
--     writeline(OUTPUT, OUTLINE);
   END IF;
 END IF;

 IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - ADR_CHNG) < t13 AND tc_en_set = '1')
 THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: ADR SETUP TO BPCLK RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
 END IF;
 IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - BE_CHNG) < t13 AND tc_en_set = '1')
 THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: BE SETUP TO BPCLK RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
 END IF;
 IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - RD_CHNG) < t29 AND tc_en_set = '1')
 THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: RD SETUP TO BPCLK RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
 END IF;
 IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - RDFIFO_CHNG) < t29 AND tc_en_set = '1')
 THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: RDFIFO SETUP TO BPCLK RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
 END IF;
 IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - WRFIFO_CHNG) < t29 AND tc_en_set = '1')
 THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: WRFIFO SETUP TO BPCLK RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
 END IF;
 IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - WR_CHNG) < t29 AND tc_en_set = '1')
 THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: WR SETUP TO BPCLK RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
 END IF;
 IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - DQ_CHNG) < t31 AND tc_en_set = '1'
     AND WR = '0')
 THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: DQ SETUP TO BPCLK RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
 END IF;

 IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - CS_CHNG) < t10a AND tc_en_set = '1')
 THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: CS SETUP TO BPCLK RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
 END IF;

  IF ((BPCLK = '1' AND BPCLK'EVENT) AND (now - PTRDY_CHNG) < t26 AND tc_en_set = '1')
  THEN
   WRITE(OUTLINE, STRING'("TIMING VIOLATION: PTRDY SETUP TO RD RISING @: "));
   write (OUTLINE, now);
--   writeline(OUTPUT, OUTLINE);
  END IF;

 
  
END PROCESS timing_checks;





END; -- behavior of amcc5933




